// -*- Mode:C++ -*-
//Header:
//File: FieldState.cc
//Title: FieldState.cc
//Author: NODA, Itsuki
//Date: 2001/03/10
//

//ModifyHistory:
// 2001/03/10: Start to create this file
//EndModifyHistory:

#include "FieldState.h"

namespace SClient {

   //======================================================================
   // Estimate 

   //--------------------------------------------------
   // estimate top

   Bool FieldState::estimate(const SeeInfo & sinfo,
			     BasePlayer & player) {
      Bool r ;

      r = estimateMyPos(sinfo,player) ;
      
      if(r) r = estimateOther(sinfo,player) ;

      return r ;
   } ;

   //--------------------------------------------------
   // estimate my position

   Bool FieldState::estimateMyPos(const SeeInfo & sinfo,
				  BasePlayer & player) {
      Bool r = estimatePosDirVelTeam2(myposdir(),posConf(),myvel(),velConf(),
				      outofFieldP(),
				      sinfo, player.side) ;
      if(r) {
	 mybdir() = myhdir() - neckAngle() ;
	 time() = sinfo.time ;
      }

      return r ;
   } ;

   //--------------------------------------------------
   // estimate other info

   Bool FieldState::estimateOther(const SeeInfo & sinfo,
				  BasePlayer & player) {
      unknownN = 0 ;

      for(UInt i = 0 ; i < sinfo.objN ; i++) {
	 const SeeInfo::Object & obj = sinfo.obj[i] ;

	 if        (obj.isBall()) {
	    ball().estimate(obj,self(),sinfo.time) ;
	 } else if (obj.isPlayer()) {
	    if        (obj.isTeammate() && obj.isUNumKnown()) {
	       teammate(obj.unum()).estimate(obj, self(), sinfo.time) ;
	    } else if (obj.isOpponent() && obj.isUNumKnown()) {
	       opponent(obj.unum()).estimate(obj, self(), sinfo.time) ;
	    } else {
	       unknown(unknownN).estimate(obj, self(), sinfo.time) ;
	       unknownN++ ;
	    }
	 } else {
	    /* do nothing for flags, goals, and lines */
	 }
      }

      return True ;
   } ;

   //--------------------------------------------------
   // Object::estimate

   Bool FieldState::Object::estimate(const SensorValue & oinfo,
				     const Object & origin,
				     const CycleStep & t) {
      time() = t ;
      svalue = oinfo ;

      Angle dir = origin.hdir() + oinfo.dir ;

      if(oinfo.know_dist() && oinfo.know_dir()) {
	 pos().x = origin.pos().x + oinfo.dist * Cos(dir) ;
	 pos().y = origin.pos().y + oinfo.dist * Sin(dir) ;
      }

      if(oinfo.know_dist_chng() && oinfo.know_dir_chng()) {
	 /* *** */
      }

      if(oinfo.know_body_dir()) {
	 bdir() = normalizeAngle(origin.hdir() + oinfo.body_dir) ;
      }

      if(oinfo.know_head_dir()) {
	 hdir() = normalizeAngle(origin.hdir() + oinfo.head_dir) ;
      }

      return True ;

   } ;

} ;
