// -*- Mode:C++ -*-
//Header:
//File: SimplePlayer.cc
//Title: Base Player for Soccer Server 7
//Author: NODA, Itsuki
//Date: 2001/03/10
//

//ModifyHistory:
// 2001/03/10: Start to create this file
//EndModifyHistory:

#include "SimplePlayer.h"

namespace SClient {

   //======================================================================
   // analyze see info

   Bool SimplePlayer::analyzeSensorSee() { 
      currentState().estimate(*this) ;
      stime() = vtime() ;
      return True ; 
   } ;

   //======================================================================
   // analyze see info

   Bool SimplePlayer::analyzeSensorHear() { 
      if(sinfo.hear.sender == HearInfo::Referee) {
	 cvPlayMode.lock() ; {
	    playmode() = sinfo.hear.aux.pmode ;
	    cvPlayMode.broadcast() ;
	 } ; cvPlayMode.unlock() ;
      }
      atime() = sinfo.time ;
      stime() = atime() ;
      return True ;
   } ;

   //======================================================================
   // analyze see info

   Bool SimplePlayer::analyzeSensorSenseBody() { 
      btime() = sinfo.time ;
      stime() = btime() ;
      return True ;
   } ;

   //======================================================================
   // analyze see info

   Bool SimplePlayer::analyzeSensorFullState() { 

      taughtState().time() = sinfo.fullstate.time ;
      taughtState().ballpos() = sinfo.fullstate.ballpos ;
      taughtState().ballvel() = sinfo.fullstate.ballvel ;
      FullStateInfo::Player * p = ITK_NULLPTR;
      for(UInt i = 0 ; i < sinfo.fullstate.playerN ; i++) {
	 if(sinfo.fullstate.player[i].side == side &&
	    sinfo.fullstate.player[i].unum == unum) {
	    p = &(sinfo.fullstate.player[i]) ;
	    break;
	 }
      }
      taughtState().myposdir() = p->posdir ;
      taughtState().myvel() = p->vel ;
      
      fstime() = sinfo.time ;
      stime() = fstime() ;
      return True ;
   } ;

   //======================================================================
   // Basic Command

   //--------------------------------------------------
   // move

   Bool SimplePlayer::move(const Flt & x, const Flt & y) { 
      return sendMove(x,y) ; 
   } ;

   //--------------------------------------------------
   // turn

   Bool SimplePlayer::turn(const Flt & power) { 
      return sendTurn(power) ; 
   } ;

   //--------------------------------------------------
   // dash

   Bool SimplePlayer::dash(const Flt & power) { 
      return sendDash(power) ; 
   } ;

   //--------------------------------------------------
   // kick

   Bool SimplePlayer::kick(const Flt & power, const Flt & dir) {
      return sendKick(power,dir) ;
   } ;

   //--------------------------------------------------
   // goalie catch

   Bool SimplePlayer::gcatch(const Flt & dir) {
      return sendCatch(dir) ;
   } ;

   //--------------------------------------------------
   // turn neck

   Bool SimplePlayer::head(const Flt & dir) {
      return sendTurnNeck(dir) ;
   } ;

   //--------------------------------------------------
   // say

   Bool SimplePlayer::say(const SubString & msg) {
      return sendSay(msg) ;
   } ;

   //======================================================================
   // low level command

   //--------------------------------------------------
   // turns

   Bool SimplePlayer::turnTo(const Flt adir) {
      return turn(normalizeAngle(adir - mybdir())) ;
   } ;

   Bool SimplePlayer::turnToRel(const Vec & rpos) {
      return turnTo(rpos.angle()) ;
   } ;

   Bool SimplePlayer::turnToAbs(const Vec & apos) {
      return turnToRel(mypos().diffTo(apos)) ;
   } ;

   //--------------------------------------------------
   // kicks

   Bool SimplePlayer::kickTo(const Flt & power, const Flt & adir) {
      return kick(power, normalizeAngle(adir - mybdir())) ;
   } ;

   Bool SimplePlayer::kickToRel(const Flt & power, const Vec & rpos) {
      return kickTo(power, rpos.angle()) ;
   } ;

   Bool SimplePlayer::kickToAbs(const Flt & power, const Vec & apos) {
      return kickToRel(power, mypos().diffTo(apos)) ;
   } ;

   //--------------------------------------------------
   // neck

   Bool SimplePlayer::headTo(const Angle & adir) {
      return head(normalizeAngle(adir - myhdir())) ;
   } ;

   Bool SimplePlayer::headToRel(const Vec & rpos) {
      return headTo(rpos.angle()) ;
   } ;

   Bool SimplePlayer::headToAbs(const Vec & apos) {
      return headToRel(mypos().diffTo(apos)) ;
   } ;

} ;
   
 
   

