// -*- Mode: c++ -*-
//Header:
//File: Buffer.h
//Author: NODA, Itsuki
//Date: 1999/05/18
//

//ModifyHistory:
// 1999/05/18: Start to create this file
// 1999/11/07: introduce Itk namespace
// 1999/11/07: Divide each class to separate file
//EndModifyHistory:

#ifndef _itk_Buffer_h_
#define _itk_Buffer_h_
//////////////////////////////////////////////////////////////////////

#include "itk/btype.h"
#include "itk/SString.h"

namespace Itk {
   
   //======================================================================
   // Buffer
   //

   class Buffer : public SubString {
      public:
	 SString original ;

	 //--------------------------------------------------
	 // constructor

	 Buffer(const UInt size = 0) { alloc(size) ; } ;
	 Buffer(SubString& org) { set(org) ; } ;

	 //--------------------------------------------------
	 // alloc

	 Bool alloc(const UInt l) ;

	 //--------------------------------------------------
	 // rewind

	 void clear() {
	    SubString::set(original.head, original.head) ;
	 } ;

	 void rewindRead() { head = original.head ; } ;
	 void rewindWrite() { clear() ; } ;

	 //--------------------------------------------------
	 // size

	 UInt size() const { return original.size() ; } ;

	 //--------------------------------------------------
	 // operator=

	 Buffer& operator=(const Buffer& src) {  return set(src) ; } ;

	 //--------------------------------------------------
	 // set

	 Buffer& set(const Buffer& src) ;

	 Buffer& set(const SubString& src) ;

	 //--------------------------------------------------
	 // used/body/rest

	 SubString used() const {
	    return SubString(original.head,head) ;
	 } ;
	 UInt usedLength() const {
	    return used().length() ;
	 } ;

	 SubString body() const {
	    return SubString(head,tail) ;
	 } ;
	 UInt bodyLength() const {
	    return length() ;
	 } ;

	 SubString rest() const {
	    return SubString(tail,original.tail) ;
	 } ;
	 UInt restLength() const {
	    return rest().length() ;
	 } ;

	 //--------------------------------------------------
	 // safety put

	 char putCharSafe(const char c) ;

	 //--------------------------------------------------
	 // describe

	 void describe(ostream& o, Bool detailp = True) const {
	    if(detailp) {
	       o << "#Buffer[<" << (void * const)(original.head) << ">-<" 
		 << (void * const)(original.tail)
		 << ">(len=" << length() << "):" 
		 << "hp=<" << (void * const)head << ">,tp=<" 
		 << (void * const)tail << ">:" ;
	    } 
	    if(isNull(original.head)) {
	       o << "(null)" ;
	    } else {
	       for(char* s = head ; s < tail ; s++) {
		  o << *s ;
	       }
	    }
	    if(detailp) {
	       o << "]" << endl ;
	    }
	 } ;

	 friend ostream& operator<< (ostream& o, const Buffer& b) {
	    b.describe(o,False) ;
	    return o ;
	 } ;

   } ;
} ;


//////////////////////////////////////////////////////////////////////
#endif
