// -*- Mode:C++ -*-
//Header:
//File: sclient1.cc
//Title: Simple Client 1
//Author: NODA, Itsuki
//Date: 2001/03/31
//

//ModifyHistory:
// 2001/03/31: Start to create this file for soccer server manual
//EndModifyHistory:

//Usage:
//	% sclient1 [TEAMNAME [SERVERHOST [SERVERPORT]]]
// 
//EndUsage:



#include "BasePlayer.h"
#include "estimatePos.h"
#include "FieldState.h"

using namespace SClient ;
using namespace Itk ;

//======================================================================
// My Player

class MyPlayer : public BasePlayer {
   public:
      virtual const char * _classname() const { return "MyPlayer";};

      FieldState field ;	/* estimated field state */
      CycleStep  stime ;	/* server timestep */

      //--------------------------------------------------
      // run loop (main loop)

      Bool run(char * shost, UInt sport, char * teamname) ;

      //--------------------------------------------------
      // Action Part

      void doAction() ;

      //--------------------------------------------------
      // Sensing Part

      void doSensing() ;

} ;

//----------------------------------------------------------------------
// run loop (main loop)

Bool MyPlayer::run(char * shost, UInt sport, char * teamname) {

   //----------------------------------------
   // initialize socket connection and send init command

   init(shost,sport,teamname) ;
	 
   //----------------------------------------
   // move to random position

   Flt x = fltRand(-50,-1) ;
   Flt y = fltRand(-30,30) ;
   sendMove(x,y) ;

   //----------------------------------------
   // enter main loop

   SocketReadSelector selector ;

   TimeVal nexttic ;  // indicate the timestamp for next command send
   nexttic.update() ; // set nexttic to the current time.

   while(True) {

      //-------------------------------------------------
      // setup selector

      selector.clear() ;
      selector.set(socket) ;
	    
      //-------------------------------------------------
      // wait socket input or timeout (100ms) ;

      Int r = selector.selectUntil(nexttic) ;

      if(r == 0) { // in the of timeout. (no sensor input)
	 doAction() ;  // enter action part
	 nexttic += TimeVal(0,100,0) ; // increase nexttimetic 100ms
      } else {  // got some input
	 doSensing() ; // enter sensor part
      }
   }
   return True ; // never reach here
}

//----------------------------------------------------------------------
// Action Part  (only chasing the ball and kick to the opponent goal)

void MyPlayer::doAction() {

   //------------------------------
   // if estimated position is too old, 
   // look around by (turn 60)

   if(!(field.ball().isNewer(stime.count - 10) &&
	field.time().count > stime.count - 10)) {
      sendTurn(60) ;
      return ;
   }

   Vec posdiff = field.ballpos() - field.mypos() ;
   if(posdiff.norm() > 1.0) {
      //------------------------------
      // if ball is far, 
      // aproach to the ball

      Flt dirdiff = posdiff.angle() - field.mybdir() ;
      if(Abs(dirdiff) > 5.0) {
	 //--------------------
	 // if the player is not facing to the ball,
	 // turn to the ball

	 sendTurn(dirdiff) ;

      } else {
	 //--------------------
	 // if the player is facing to the ball,
	 // dash to the ball with power 50

	 sendDash(50) ;
      }
   } else {
      //------------------------------
      // if ball is near enough
      // kick it to the opponent goal (with power 100)

      Vec goalpos(54,0) ;  
      Vec dpos = goalpos - field.mypos() ;
      Flt dir = dpos.angle() - field.mybdir() ;

      sendKick(100, dir) ;
   }

   return ;
} ;

//----------------------------------------------------------------------
// Sensing Part

void MyPlayer::doSensing() {
   
   //------------------------------
   // receive a sensor info

   Bool r = cycleSensor() ;

   //------------------------------
   // if the data is received successfully, then analyze the info

   if(r) {
      switch(sinfo.type) {
	 case SensorInfo::T_See : {
	    field.estimate(*this) ;
	    stime = sinfo.time ;
	    break ;
	 }
	 case SensorInfo::T_Hear : { 
	    /* enter code for analyze hear info */
	    break ; 
	 }
	 case SensorInfo::T_SenseBody : { 
	    /* enter code for analyze sensebody info */
	    break ; 
	 }
	 case SensorInfo::T_FullState : { 
	    /* enter code for analyze fullstate info */
	    break ; 
	 }
	 default : {
	 }
      }
   }
   
} ;
      
//======================================================================
// Main

int main(int argc, char** argv) {

   char * shost = "localhost" ;
   UInt sport = 6000 ;
   char * teamname = "MyTeam" ;

   if(argc > 1) teamname = argv[1] ;
   if(argc > 2) shost = argv[2] ;
   if(argc > 3) sport = atoi(argv[3]) ;
   
   MyPlayer player ;
   player.run(shost,sport,teamname) ;
   return 0 ;
} ;
   

